

class Units(object):
    """
    Unit information about various aspects of the forecast

    Attributes:
        temperature: Temperature degree units, F for Fahrenheit C for Celsius
            (string)
        pressure: The units of barometric pressure, in pounds per square inch
            or millibars (string).
        distance: The units of distance in miles of km (string).
        speed: The units of speed in mph or kph (string)
    """

    def __init__(self, units):
        self.temperature = units['temperature']
        self.pressure = units['pressure']
        self.distance = units['distance']
        self.speed = units['speed']




from dateutil import parser


class Condition(object):
    """
    The current weather conditions.

    Attributes:
        code: The condition code for the forecast. The possible values for the
            code are described at the following URL (integer).
            http://developer.yahoo.com/weather/#codes
            If no code is found code will be None.
        date: The date and time the forecast was posted, time-zone unaware
            (datetime).
        temperature: The current temperature in the units specified in the
            temperature variable of the Units class (integer). If no value for the
            temperature is found, temperature will be None.
        text: A textual description of the conditions. E.g. Partly Cloudy
            (string).
    """

    def __init__(self, condition):
        try:
            self.temperature = int(condition['temp'])
        except ValueError:
            self.temperature = None
        self.date = parser.parse(condition['date'], ignoretz=True)
        self.text = condition['text']
        try:
            self.code = int(condition['code'])
        except ValueError:
            self.code = None



from astronomy import Astronomy
from atmosphere import Atmosphere
from condition import Condition
from dateutil import parser
from forecast import Forecast
from image import Image
from location import Location
from units import Units
from wind import Wind
import urllib2
import xml.etree.ElementTree as etree


class Response(object):
    """
    The base class that handles interacting with the Yahoo weather API.

    This class is the main class you will be using to interact with the API.
    You must call the class with a user agent (as a string) and the WOEID of
    the location you want the weather from (as an integer or long). You can
    find the WOEID of the location you want the weather from at the
    following URL.

    http://woeid.rosselliot.co.nz/

    You can also call the Response class with a keyword argument metric. This
    will determine whether your response is in metric or imperial units. By
    default metric is set to true.

    Attributes:
        astronomy: An instance of the astronomy.Astronomy class representing
            forecast information about the current astronomical conditions.
        atmosphere: An instance of the atmosphere.Atmosphere class representing
            forecast information about the current atmospheric conditions.
        condition: An instance of the condition.Condition class representing
            current weather conditions.
        conditions_title: The forecast title and time as a string.
        description: The description of the response RSS feed as a string.
        forecasts: A list of forecast.Forecast objects, one for each forecast
            in the RSS feed.
        image: An instance of the image.Image class representing the image in
            the RSS feed.
        language: The language of the response RSS feed as a string.
        last_bulid_date: The last time the feed was updated as a non time-zone
            aware datetime.
        latitude: The latitude of the location as a float.
        link: The URL of the weather page of the forecast for this location as
            a string.
        location: An instance of the location.Location class representing the
            location of the forecast.
        location: An instance of the location.Location class representing the
            location of the forecast.
        longitude: The longitude of the location as a float.
        metric: A boolean indicating if you want metric units or not.
        pub_date: The date and time the forecast was posted as a time-zone
            unaware datetime.
        title: The title of the response RSS feed as a string.
        today: An instance of the forecast.Forecast class representing today's
            forecast.
        tomorrow: An instance of the forecast.Forecast class representing
            tomorrow's forecast.
        ttl: An integer representing the Time To Live in minutes. This is how
            long the feed should be cached.
        units: An instance of the units.Units class representing the units for
            various aspects of the forecast.
        user_agent: The user agent you wish to supply to the Yahoo weather API.
        wind: An instance of the wind.Wind class representing forecast
            information about the wind.
        woeid: The WOEID of the location you want the weather from.
    """

    base_url = 'http://weather.yahooapis.com/forecastrss'
    weather_namespace = '{http://xml.weather.yahoo.com/ns/rss/1.0}'
    geo_namespace = '{http://www.w3.org/2003/01/geo/wgs84_pos#}'

    def __init__(self, user_agent, woeid, metric=True):
        self.user_agent = user_agent
        assert isinstance(woeid, (int, long))
        self.woeid = str(woeid)
        self.metric = metric
        self._channel = self._get_xml(metric)
        self.title = self._parse_text('title')

        if self.title == 'Yahoo! Weather - Error':
            raise Exception('City not found for WOEID {0}'.format(self.woeid))

        self.description = self._parse_text('description')
        self.language = self._parse_text('language')
        self.ttl = int(self._parse_text('ttl'))
        self.last_build_date = self._parse_last_build_date()
        self.forecasts = self._parse_forecasts()
        self.conditions_title = self._channel.find('./item/title').text
        self.latitude = float(self._parse_text('lat', self.geo_namespace))
        self.longitude = float(self._parse_text('long', self.geo_namespace))
        self.link = self._parse_text('link')
        self.pub_date = parser.parse(self._parse_text('pubDate'))
        self.today = self.forecasts[0]
        self.tomorrow = self.forecasts[1]
        self.image = self._parse_image()

        self.location = self._parse_attributes(
            'location',
            Location,
            self.weather_namespace
        )

        self.units = self._parse_attributes(
            'units',
            Units,
            self.weather_namespace
        )

        self.wind = self._parse_attributes(
            'wind',
            Wind,
            self.weather_namespace
        )

        self.atmosphere = self._parse_attributes(
            'atmosphere',
            Atmosphere,
            self.weather_namespace
        )

        self.astronomy = self._parse_attributes(
            'astronomy',
            Astronomy,
            self.weather_namespace
        )

        self.condition = self._parse_attributes(
            'condition',
            Condition,
            self.weather_namespace
        )

    def _get_xml(self, metric):
        """Returns the channel element of the RSS feed"""
        self._opener = urllib2.build_opener()
        self._opener.addheaders = [('User-agent', self.user_agent)]

        if metric:
            url = self.base_url + '?w={0}&u=c'.format(self.woeid)
        else:
            url = self.base_url + '?w={0}'.format(self.woeid)

        return etree.parse(
            self._opener.open(url)
        ).getroot()[0]

    def _parse_text(self, element_name, namespace=''):
        """
        Returns the text, as a string, of the specified element in the specified
        namespace of the RSS feed.

        Takes element_name and namespace as strings.
        """
        try:
            text = self._channel.find('.//' + namespace + element_name).text
        except AttributeError:
            raise Exception(
                'Element, {0} not found in RSS feed'.format(element_name)
            )

        return text

    def _parse_attributes(self, element_name, package_class, namespace=''):
        """
        Returns an instance of the package_class instantiated with a
        dictionary of the attributes from element_name in the specified
        namespace of the RSS feed.
        """
        return package_class(
            self._channel.find(
                './/{0}{1}'.format(namespace, element_name)
            ).attrib
        )

    def _parse_forecasts(self):
        """
        Returns a list of instances of the forecast.Forecast class. Each
        instance of the class is instantiated with the attributes of the
        forecast elements in the RSS feed.
        """
        forecasts = self._channel.findall(
            './/{0}{1}'.format(self.weather_namespace, 'forecast')
        )
        return [Forecast(forecast.attrib) for forecast in forecasts]

    def _parse_image(self):
        """
        Returns an instance of the image.Image class for the RSS feed.
        """
        image = {
            'title': self._channel.find('./image/title').text,
            'width': int(self._channel.find('./image/width').text),
            'height': int(self._channel.find('./image/height').text),
            'link': self._channel.find('./image/link').text,
            'url': self._channel.find('./image/url').text
        }

        return Image(image)

    def _parse_last_build_date(self):
        """
        Returns the last build date of the RSS feed as datetime.datetime
        object. Returned datetime is not time-zone aware
        """
        date = self._channel.find('lastBuildDate').text
        date = parser.parse(date, ignoretz=True)
        return date





class Atmosphere(object):
    """
    Summary of the current atmospheric conditions.

    Attributes:
        humidity: Current humidity as a percentage (float). If a value for the
            humidity is not found, humidity will be None.
        visibility: Visibility in units specified in the distance variable of
            the Units class (float). If a value for visibility is not found,
            visibility will be None.
        pressure: Barometric pressure in units specified in the pressure
            variable of the Units class (float). If a value for barometric
            pressure is not found, pressure will be None.
        rising: State of the barometric pressure. 0 for steady, 1 for rising,
            or 2 for falling (integer). If a value for the state of barometric
            pressure is not found, rising will be None.
    """

    def __init__(self, atmosphere):
        try:
            self.humidity = float(atmosphere['humidity'])
        except ValueError:
            self.humidity = None
        try:
            self.visiblity = float(atmosphere['visibility'])
        except ValueError:
            self.visiblity = None
        try:
            self.pressure = float(atmosphere['pressure'])
        except ValueError:
            self.pressure = None
        try:
            self.rising = int(atmosphere['rising'])
        except ValueError:
            self.rising = None





class Astronomy(object):
    """
    Forcast information about the current astronomic conditions

    Attributes:
        sunrise: Today's sunrise time. The time is local time and in the format
            h:mm am/pm
        sunset: Today's sunset time. The time is local time and in the format
            h:mm am/pm
    """

    def __init__(self, astronomy):
        self.sunrise = astronomy['sunrise']
        self.sunset = astronomy['sunset']




from dateutil import parser


class Forecast(object):
    """
    The weather forecast for a specific day.

    Attributes:
        day: The day of the week to which the forecast applies. Possible values
            are Mon, Tue, Wed, Thu, Fri, Sat, Sun (string).
        date: The date to which to forecast applies, time-zone unaware (datetime)
        low: The forecasted low temperature for the day in units specified in
           the temperature variable of the Units class (integer). If a value for
           forecasted low temperature is not found, low will be None.
        high: The forecasted high temperature for the day in units specified in
           the temperature variable of the Units class (integer). If a value for
           forecasted high temperature is not found, high will be None.
        text: A textual description of conditions (string).
    """

    def __init__(self, forecast):
        self.day = forecast['day']
        self.date = parser.parse(forecast['date'], ignoretz=True)
        try:
            self.low = int(forecast['low'])
        except ValueError:
            self.low = None
        try:
            self.high = int(forecast['high'])
        except ValueError:
            self.high = None
        self.text = forecast['text']





class Location(object):
    """
    The location of the forecast

    Attributes:
        city: The name of the city (string).
        country: The two character country code for the country (string).
        region: The name of the state, territory, or region if given. Empty
            string if not given (string).
    """

    def __init__(self, location):
        self.city = location['city']
        self.region = location['region']
        self.country = location['country']





class Wind(object):
    """
    Current forecast information about the wind.

    Attributes:
        chill: Wind chill in degrees (integer). If a value for wind chill is not
            found, chill will be None.
        direction: Wind direction in degrees (integer). If a value for wind
            direction is not found, direction will be None.
        speed: Wind speed in units specified in the speed attribute of the
            Units class (integer). If a value for wind speed is not found, speed
            will be None.
    """

    def __init__(self, wind):
        try:
            self.chill = int(wind['chill'])
        except ValueError:
            self.chill = None
        try:
            self.direction = int(wind['direction'])
        except ValueError:
            self.direction = None
        try:
            self.speed = float(wind['speed'])
        except ValueError:
            self.speed = None

    def cardinal_direction(self):
        """
        Returns the cardinal direction of the
        wind as a string. Possible returned values are N, E, S, W, and None.

        315 degrees to 45 degrees exclusive -> N
        45 degrees to 135 degrees exclusive -> E
        135 degrees to 225 degrees exclusive -> S
        225 degrees to 315 degrees exclusive -> W

        None if no direction found.
        """
        if self.direction is None:
            return None

        if self.direction > 360 or self.direction < 0:
            raise Exception('Direction out of range')

        if (315 <= self.direction) <= 360 or 0 <= (self.direction) < 45:
            return 'N'
        elif 45 <= self.direction < 135:
            return 'E'
        elif 135 <= self.direction < 225:
            return 'S'
        elif 225 <= self.direction < 315:
            return 'W'





class Image(object):
    """
    The image use to identify the RSS feed

    Attributes:
        height: The height of the image in  pixels (int).
        link: Link to Yahoo weather.
        title: The title of the image (string).
        url: The url of the image (string).
        width: The width of the image in pixels (int).
    """

    def __init__(self, image):
        self.title = image['title']
        self.width = image['width']
        self.height = image['height']
        self.link = image['link']
        self.url = image['url']

    def as_html(self):
        """
        Returns the image of the Yahoo rss feed as an html string
        """

        return '<a href="{0}"><img height="{1}" width="{2}" src="{3}" alt="{4}"></a>'.format(
            self.link, self.height, self.width, self.url, self.title)



from distutils.core import setup

setup(
    name = 'weatherpy',
    packages = ['weatherpy'],
    version = '1.0.2',
    description = 'Python wrapper for Yahoo weather API',
    author = 'Christopher McDowell',
    author_email = 'chris.p.mcdowell@gmail.com',
    url = 'https://github.com/cmcdowell/weatherpy',
    install_requires = ['python-dateutil'],
    classifiers = [
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Development Status :: 4 - Beta',
        'Operating System :: OS Independent',
        'License :: OSI Approved :: MIT License',
        'Intended Audience :: Developers',
        'Environment :: Other Environment',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Utilities',
    ],
    long_description = """\
Weatherpy, python wrapper for Yahoo weather API
-----------------------------------------------

Weatherpy is a package that allows you to simply and easily access Yahoo's
weather API. Give weatherpy a user agent and a WOEID and weatherpy will make
accessing elements of the RSS feed simple.

example.

    import weatherpy
    r = weatherpy.Response('Bob's weather script', 44544)
    print 'Wind: ', r.wind.speed, r.units.speed, r.wind.cardinal_direction()

gives

    Wind: 6 km/h N

Weatherpy is only tested with python 2.5+. Weatherpy is not compatible wihth
python 3.
""")
